from Products.Plonelicious.config import UNIQUE_TOOL_ID, PRODUCT_NAME, SKIN_NAMES, SKIN_DIRS, CONFIGLETS, PRODUCT_PROPERTIES, ACTION_ICONS
from StringIO import StringIO
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.DirectoryView import createDirectoryView

################ INSTALL FUNCTIONS############################
def install_props(self, out):
    """ Install properties """
    if not hasattr(self.portal_properties, 'plonelicious_properties'):
        self.portal_properties.addPropertySheet('plonelicious_properties', 'Plonelicous properties')

    props = self.portal_properties.plonelicious_properties

    for prop_id, prop_type, prop_value in PRODUCT_PROPERTIES:
        if not hasattr(props, prop_id):
            props._setProperty(prop_id, prop_value, prop_type)
    out.write('Properties added.\n')

def install_tools(self, out):
    if not hasattr(self, UNIQUE_TOOL_ID):
        addTool = self.manage_addProduct[PRODUCT_NAME].manage_addTool
        addTool(PRODUCT_NAME + ' Tool')
        out.write('Added "%s" Tool\n' % PRODUCT_NAME)
       
def install_actions(self, out):
    at = getToolByName(self, "portal_actions")
    at.manage_aproviders(UNIQUE_TOOL_ID, add_provider=1)
    out.write('Installed actions for "%s"\n' % UNIQUE_TOOL_ID)
    action_icons_tool = getToolByName(self, 'portal_actionicons')
    for category, config in ACTION_ICONS.items():
        for icon_id, info in config.items():
            if action_icons_tool.queryActionIcon(category, icon_id, None) is None:
                action_icons_tool.addActionIcon(category, icon_id,
                                 info[0], info[1])
                out.write('Installed action icon '
                               'for %s.\n' % info[1])
            else:
                out.write('Action Icon for %s '
                               'was already Installed.\n'
                               % info[1])
    
def install_skins(self, out):

    skinsTool = getToolByName(self, 'portal_skins')
        
    for name, location in zip(SKIN_NAMES, SKIN_DIRS):
        if name not in skinsTool.objectIds():
            createDirectoryView(skinsTool, location, name)
            out.write('Added "%s" directory view to portal_skins\n' % name)

    skins = skinsTool.getSkinSelections()
    for skin in skins:
        path = skinsTool.getSkinPath(skin)
        path = [ p.strip() for p in path.split(',') ]
        for name, location in zip(SKIN_NAMES, SKIN_DIRS):
            if name not in path:
                path.insert(path.index('custom')+1, name)

                p = ", ".join(path)
                skinsTool.addSkinSelection(skin, p)
                out.write('Added "%s" to "%s" skins\n' % (name, skin))
            else:
                out.write('Skipping "%s" skin\n' % skin)

def install_configlets(self, out):
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        for conf in CONFIGLETS:
            configTool.registerConfiglet(**conf)
            out.write('Added configlet %s\n' % conf['id'])         

################### UNINSTALL FUNCTIONS ########################

def uninstall_skins(self, out):
    # TODO: remove action and action icon
    skinsTool = getToolByName(self, 'portal_skins')
    skins = skinsTool.getSkinSelections()
    for skin in skins:
        path = skinsTool.getSkinPath(skin)
        path = [itm.strip() for itm in path.split(',')]
        path = ','.join([n for n in path if not n in SKIN_NAMES])
        skinsTool.addSkinSelection(skin, path)
    out.write('Removed skin(s).\n')

def uninstall_configlets(self, out):
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        for conf in CONFIGLETS:
            out.write('Removing configlet %s\n' % conf['id'])
            configTool.unregisterConfiglet(conf['id'])

###############################################################

def install(self):
    out = StringIO()
    install_props(self, out)
    install_tools(self, out)
    install_actions(self, out)
    install_skins(self, out)
    install_configlets(self, out)
    out.write('Install complete.\n')    
    return out.getvalue()


def uninstall(self):
    out=StringIO()
    uninstall_skins(self, out)
    at = getToolByName(self, "portal_actions")
    at.deleteActionProvider(UNIQUE_TOOL_ID)
    uninstall_configlets(self, out)
    out.write('Uninstall complete.\n')    
